import argparse
from argparse import Action


class CustomHelpFormatter(argparse.HelpFormatter):
    def _format_action_invocation(self, action: Action) -> str:
        if not action.option_strings:
            default = self._get_default_metavar_for_positional(action)
            (metavar,) = self._metavar_formatter(action, default)(1)
            return metavar
        else:
            parts = []

            if action.nargs == 0:
                parts.extend(action.option_strings)
            else:
                default = self._get_default_metavar_for_optional(action)
                args_string = self._format_args(action, default)
                for option_string in action.option_strings:
                    parts.append(option_string)

                return "%s %s" % (", ".join(parts), args_string)

            return ", ".join(parts)

    def _get_default_metavar_for_optional(self, action):
        return action.dest.upper()



import os

from .get import get_bucket_layout
from .generator import generate
from .upload import upload


class _Builder:
    """
    Takes all the classes and combines them to create and upload files to s3 bucket
    """

    def __init__(self) -> None:
        self.packages = []

    def _run(self, bucket):
        """
        starting point of class
        """
        self.bucket_name = bucket
        ### CHECK BUCKET NAME HERE!! SHOULD match (bucket.com) or (xxx.bucket.com) ###
        ### if s3:// given remove before continuing ###
        self.layout = get_bucket_layout(self.bucket_name)
        self._bucket_loop()

    def _create_key(self, path):
        """
        takes a path as input and joins it with "index.html" to create the
        desired s3 bucket key.
        """
        return os.path.join(path, "index.html")

    def _bucket_loop(self):
        """
        Loops through the dictionary returned by get_bucket_layout().
        Inside the loop we call the generate() method te create the
        desired HTML for the index page. This recurses into all directories
        inside the bucket. Every directory is given an index.html file that
        has been programmatically created from names of objects inside the
        bucket which is then uploaded to the bucket.
        """
        # loops through each package
        # append self.packages with each package name

        for name in self.layout.keys():
            self.packages.append(name)

            # generate package index
            self.gen_package = generate(
                title=name,
                header=name,
                links=self.layout[name]["files"],
            )

            # get bucket path
            upload(self.bucket_name, self.gen_package, name)
            # print(self.gen_package)

            # generate version index
            # for version in self.layout[name]["versions"]:
            #     self.files = [
            #         file for file in self.layout[name]["files"] if version in file
            #     ]

            #     self.gen_version = generate(
            #         title=name,
            #         header=f"""{name} - {version}""",
            #         links=self.files,
            #     )

            #     # get bucket path
            #     upload(self.bucket_name, self.gen_version, os.path.join(name, version))
            #     # print(self.gen_version)

        # generate root index
        self.gen_root = generate(
            title=self.bucket_name,
            header=self.bucket_name,
            links=self.packages,
        )
        # get bucket path
        upload(self.bucket_name, self.gen_root, "")
        # print(self.gen_root)


_builder = _Builder()


def build(bucket):
    """
    The public method for the _Builder class
    """
    return _builder._run(bucket)


if __name__ == "__main__":
    build("pip.trainland.io")



import os
import tempfile
import boto3


class _Upload:
    def __init__(self) -> None:
        pass

    def _run(self, bucket, file: str, s3_path: str):
        self.file = file
        self.s3_path = s3_path
        self.client = boto3.resource("s3")
        self.bucket = self.client.Bucket(name=bucket)

        self._create_file()

    def _create_file(self):
        with tempfile.TemporaryDirectory() as tmp:
            filepath = os.path.join(
                tmp,
                "index.html",
            )
            file = open(filepath, "w")
            with file as f:
                f.write(self.file)

            self._upload(filepath)

    def _upload(self, index):
        key = os.path.join(self.s3_path, "index.html")
        # print(key)
        self.bucket.upload_file(
            Filename=index, Key=key, ExtraArgs={"ContentType": "text/html"}
        )


_uploader = _Upload()


def upload(bucket, file, s3_path):
    return _uploader._run(bucket, file, s3_path)


if __name__ == "__main__":
    upload(
        "pip.trainland.io", "/home/mantix/prog/python/s3-index-gen/test.txt", "fake/"
    )



import argparse

from .build import build
from .parser_formatter import CustomHelpFormatter


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        prog="s3indexer", formatter_class=CustomHelpFormatter
    )
    subparsers = parser.add_subparsers(metavar="command", dest="command")

    # create the run sub-command
    subparser_run = subparsers.add_parser("run", help="run the whole shebang")
    subparser_run.add_argument(
        "bucket", help="name of bucket in which to create index files"
    )
    subparser_run.add_argument(
        "--local",
        action="store_true",
        default=False,
        help="generate index files locally",
    )
    subparser_run.add_argument(
        "--dry-run",
        action="store_true",
        default=False,
        dest="dryrun",
        help="print bucket instead of running build",
    )

    parser.add_argument

    args = parser.parse_args()

    match args.command:
        # sub-command [run]
        case "run":
            ### add a confirmation before running buld()
            if not type(args.bucket) == str:
                raise TypeError("bucket must be a string")

            if "http://" in args.bucket or "https://" in args.bucket:
                raise ValueError("name of bucket expected, got bucket url")

            if args.dryrun:
                print("--DRY-RUN", "|", args.bucket)

            # if args.local:
            #     build(args.bucket)

            build(args.bucket)

            ### add [run --version] argument which creates an index for a single version instead
            ### of rewriting all the indexes. should save on bucket read/writes.
            # if --version:
            ##  require --title --header --links
            #   match "data":
            #     case not "title":
            #       raise "title needed when using --version"
            #     ...

            #   generate(args.title, args.header, args.links)

        ### add new subcommand: [gather]                                     >
        ### takes (path/to/pyproject.toml) as argument                       > maybe different module??
        ### gets package name and version from toml file for use in pipeline >



class _HtmlGenerator:
    """
    This class is used to programatically create simple html files
    """

    def __init__(self) -> None:
        pass

    def _run(self, title: str, header: str, links: list):
        """
        starting method
        """
        self.title = f"""<title>{title}</title>"""
        self.header = f"""<h1>{header}</h1>"""
        self.links = links

        self.index = self._generate_index()

        return self.index

    def _generate_index(self):
        newline = "\n    "
        link_gen = f"""{newline.join(f'<a href="{value}">{value}</a><br />' for value in self.links)}"""

        scaffold = f"""
<html>
  <head>
    {self.title}
  </head>
  <body>
    {self.header}
    <br />
    {link_gen}
  </body>
</html>
"""

        return scaffold


_generator = _HtmlGenerator()


def generate(title: str, header: str, links: list):
    """
    this is the public method for the _HtmlGenerator class
    """
    return _generator._run(title, header, links)



import boto3


class _GetBucketLayout:
    def __init__(self) -> None:
        # set boto3 to use the s3 resource
        self.client = boto3.resource("s3")

    def _start(self, bucket):
        # setup s3 bucket object
        self.bucket = self.client.Bucket(name=bucket)

        # get packages from s3 bucket and turn into a dictionary
        self.container = self._recurse_bucket()

        return self.container

    def _recurse_bucket(self) -> dict:
        container = {}

        for obj in self.bucket.objects.all():
            # simple/0.0.1/somefile.whl
            key = obj.key

            # ignores any object that contains an index.html
            if not "index.html" in str(obj):
                # creates 3 parts to each path (project, version, files)
                # project, version, file = key.split("/")
                project, file = key.split("/")

                # if not project in container:
                #     container[project] = {"versions": [version], "files": [file]}
                # else:
                #     if version not in container[project]["versions"]:
                #         container[project]["versions"].append(version)

                #     if file not in container[project]["files"]:
                #         container[project]["files"].append(file)

                if not project in container:
                    container[project] = {"files": [file]}
                else:
                    if file not in container[project]["files"]:
                        container[project]["files"].append(file)

        return container


_collecter = _GetBucketLayout()


def get_bucket_layout(bucket):
    return _collecter._start(bucket)


if __name__ == "__main__":
    container = get_bucket_layout("pip.trainland.io")
    for key, value in container.items():
        print(f"{key}: {value}")
